@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,6 +33,61 @@
   #define BUFSIZE (64 * 1024)
 #endif
 
+char* abs_path(const char *path, const char *base_dir)
+{
+	if (path == NULL) {
+		return NULL;
+	} else if (path[0] == '/') {
+		return strdup(path);
+	} else {
+		char *full_path;
+		if (base_dir == NULL) {
+			char *cwd = realpath("./", NULL);
+			full_path = sprintf_alloc("%s/%s", cwd, path);
+			free(cwd);
+		} else {
+			full_path = sprintf_alloc("%s/%s", base_dir, path);
+		}
+		return full_path;
+	}
+}
+
+bool same_path(const char *path1, const char *path2)
+{
+	bool equal = false;
+	int err = 0;
+
+	struct stat sb1;
+	if (stat(path1, &sb1) == 0) {
+		struct stat sb2;
+		if (stat(path2, &sb2) == 0) {
+			if (sb1.st_dev == sb2.st_dev &&
+			    sb1.st_ino == sb2.st_ino) {
+				equal = true;
+			}
+		} else {
+			err = errno;
+		}
+	} else {
+		err = errno;
+	}
+
+	if (err != 0) {
+		// Can't compare real absolute paths, as stat() failed already. Try the best.
+		char *full_path1 = abs_path(path1, NULL);
+		char *full_path2 = abs_path(path2, NULL);
+
+		if (strcmp(full_path1, full_path2) == 0) {
+			equal = true;
+		}
+
+		free(full_path1);
+		free(full_path2);
+	}
+
+	return equal;
+}
+
 static int remove_file(const char *path, const struct stat *stat, int type, struct FTW *ftw)
 {
 	(void)stat;
@@ -151,7 +206,7 @@ int copy_file(const char *dest, const char *src)
 
 	FILE *from = fopen(src, "r");
 	if (from == NULL) {
-		ret = knot_map_errno();
+		ret = errno == ENOENT ? KNOT_EFILE : knot_map_errno();
 		goto done;
 	}
 
