@@ -124,6 +124,8 @@ static int share_or_generate_key(kdnssec_ctx_t *ctx, kdnssec_generate_flags_t fl
 	int ret = kasp_db_get_policy_last(ctx->kasp_db, ctx->policy->string,
 	                                  &borrow_zone, &borrow_key);
 	if (ret != KNOT_EOK && ret != KNOT_ENOENT) {
+		free(borrow_zone);
+		free(borrow_key);
 		return ret;
 	}
 
@@ -395,8 +397,8 @@ static roll_action_t next_action(kdnssec_ctx_t *ctx, zone_sign_roll_flags_t flag
 				}
 				break;
 			case DNSSEC_KEY_STATE_RETIRE_ACTIVE:
-				if (key->timing.retire == 0 && key->timing.post_active == 0) { // this shouldn't normally happen
-					// when a KSK is retire_active, it has already retire or post_active timer set
+				if (key->timing.retire == 0 && key->timing.post_active == 0 && key->timing.remove == 0) { // this shouldn't normally happen
+					// when a KSK is retire_active, it has already some following timer set
 					keytime = ksk_retire_time(key->timing.retire_active, ctx);
 					restype = RETIRE;
 				}
