@@ -217,6 +217,9 @@ int zone_update_from_differences(zone_update_t *update, zone_t *zone, zone_conte
 			return ret;
 		}
 		log_zone_info(zone->name, "automatic SOA serial increment");
+		log_zone_notice(zone->name, "automatic SOA increment is deprecated and will "
+		                            "result in error in newer versions, configure "
+		                            "'zonefile-load: difference-no-serial' instead");
 	}
 
 	update->init_cont = new_cont;
@@ -898,22 +901,14 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 		return ret;
 	}
 
-	/* Check if the zone was re-signed upon zone load to ensure proper flush
-	 * even if the SOA serial wasn't incremented by re-signing. */
 	val = conf_zone_get(conf, C_DNSSEC_SIGNING, update->zone->name);
-	bool dnssec = conf_bool(&val);
-
-	if (dnssec) {
-		update->zone->zonefile.resigned = true;
-
-		if (zone_is_slave(conf, update->zone)) {
-			ret = zone_set_lastsigned_serial(update->zone,
-			                                 zone_contents_serial(update->new_cont));
-			if (ret != KNOT_EOK) {
-				log_zone_warning(update->zone->name,
-				                 "unable to save lastsigned serial, "
-				                 "future transfers might be broken");
-			}
+	if (conf_bool(&val) && zone_is_slave(conf, update->zone)) {
+		ret = zone_set_lastsigned_serial(update->zone,
+		                                 zone_contents_serial(update->new_cont));
+		if (ret != KNOT_EOK) {
+			log_zone_warning(update->zone->name,
+			                 "unable to save lastsigned serial, "
+			                 "future transfers might be broken");
 		}
 	}
 
