@@ -21,11 +21,14 @@
 #include <string.h>
 
 #include "contrib/base64url.h"
+#include "contrib/macros.h"
 #include "contrib/url-parser/url_parser.h"
 #include "libknot/errcode.h"
 #include "utils/common/https.h"
 #include "utils/common/msg.h"
 
+#define is_read(ctx) (ctx->stream == -1)
+
 int https_params_copy(https_params_t *dst, const https_params_t *src)
 {
 	if (dst == NULL || src == NULL) {
@@ -137,8 +140,7 @@ static ssize_t https_recv_callback(nghttp2_session *session, uint8_t *data, size
 
 	ssize_t ret = 0;
 	while ((ret = gnutls_record_recv(ctx->tls->session, data, length)) <= 0) {
-		if (!ctx->read) { //Unblock `nghttp2_session_recv(nghttp2_session)`
-			ctx->read = true;
+		if (is_read(ctx)) { //Unblock `nghttp2_session_recv(nghttp2_session)`
 			return NGHTTP2_ERR_WOULDBLOCK;
 		}
 		if (ret == 0) {
@@ -164,9 +166,20 @@ static int https_on_data_chunk_recv_callback(nghttp2_session *session, uint8_t f
 
 	https_ctx_t *ctx = (https_ctx_t *)user_data;
 	if (ctx->stream == stream_id) {
-		memcpy(ctx->recv_buf, data, len);
-		ctx->recv_buflen = len;
-		ctx->read = false;
+		int cpy_len = MIN(len, ctx->recv_buflen);
+		memcpy(ctx->recv_buf, data, cpy_len);
+		ctx->recv_buf += cpy_len;
+		ctx->recv_buflen -= cpy_len;
+	}
+	return KNOT_EOK;
+}
+
+static int https_on_stream_close_callback(nghttp2_session *session, int32_t stream_id, uint32_t error_code, void *user_data)
+{
+	assert(user_data);
+
+	https_ctx_t *ctx = (https_ctx_t *)user_data;
+	if (ctx->stream == stream_id) {
 		ctx->stream = -1;
 	}
 	return KNOT_EOK;
@@ -236,6 +249,7 @@ int https_ctx_init(https_ctx_t *ctx, tls_ctx_t *tls_ctx, const https_params_t *p
 	nghttp2_session_callbacks_set_recv_callback(callbacks, https_recv_callback);
 	nghttp2_session_callbacks_set_on_data_chunk_recv_callback(callbacks, https_on_data_chunk_recv_callback);
 	nghttp2_session_callbacks_set_on_header_callback(callbacks, https_on_header_callback);
+	nghttp2_session_callbacks_set_on_stream_close_callback(callbacks, https_on_stream_close_callback);
 
 	int ret = nghttp2_session_client_new(&(ctx->session), callbacks, ctx);
 	if (ret != 0) {
@@ -252,7 +266,7 @@ int https_ctx_init(https_ctx_t *ctx, tls_ctx_t *tls_ctx, const https_params_t *p
 	ctx->params = *params;
 	ctx->authority = (tls_ctx->params->hostname) ? strdup(tls_ctx->params->hostname) : NULL;
 	ctx->path = strdup((ctx->params.path) ? ctx->params.path : (char *)default_path);
-	ctx->read = true;
+	ctx->stream = -1;
 
 	return KNOT_EOK;
 }
@@ -404,6 +418,23 @@ int https_ctx_connect(https_ctx_t *ctx, const int sockfd, struct sockaddr_storag
 	return KNOT_EOK;
 }
 
+static int https_send_dns_query_common(https_ctx_t *ctx, nghttp2_nv *hdrs, size_t hdrs_len, nghttp2_data_provider *data_provider)
+{
+	assert(hdrs != NULL && hdrs_len > 0);
+
+	ctx->stream = nghttp2_submit_request(ctx->session, NULL, hdrs, hdrs_len,
+	                                     data_provider, NULL);
+	if (ctx->stream < 0) {
+		return KNOT_NET_ESEND;
+	}
+	int ret = nghttp2_session_send(ctx->session);
+	if (ret != 0) {
+		return KNOT_NET_ESEND;
+	}
+
+	return KNOT_EOK;
+}
+
 static int https_send_dns_query_get(https_ctx_t *ctx)
 {
 	const size_t dns_query_len = strlen(ctx->path) +
@@ -428,18 +459,8 @@ static int https_send_dns_query_get(https_ctx_t *ctx)
 		MAKE_STATIC_NV("accept", "application/dns-message"),
 	};
 
-	ctx->stream = nghttp2_submit_request(ctx->session, NULL, hdrs,
-	                                     sizeof(hdrs) / sizeof(*hdrs),
-	                                     NULL, NULL);
-	if (ctx->stream < 0) {
-		return KNOT_NET_ESEND;
-	}
-	ret = nghttp2_session_send(ctx->session);
-	if (ret != 0) {
-		return KNOT_NET_ESEND;
-	}
-
-	return KNOT_EOK;
+	return https_send_dns_query_common(ctx, hdrs, sizeof(hdrs) / sizeof(*hdrs),
+	                                   NULL);
 }
 
 static ssize_t https_send_data_callback(nghttp2_session *session, int32_t stream_id,
@@ -485,18 +506,8 @@ static int https_send_dns_query_post(https_ctx_t *ctx)
 		.read_callback = https_send_data_callback
 	};
 
-	ctx->stream = nghttp2_submit_request(ctx->session, NULL, hdrs,
-	                                     sizeof(hdrs) / sizeof(nghttp2_nv),
-	                                     &data_provider, NULL);
-	if (ctx->stream < 0) {
-		return KNOT_NET_ESEND;
-	}
-	int ret = nghttp2_session_send(ctx->session);
-	if (ret != 0) {
-		return KNOT_NET_ESEND;
-	}
-
-	return KNOT_EOK;
+	return https_send_dns_query_common(ctx, hdrs, sizeof(hdrs) / sizeof(*hdrs),
+	                                   &data_provider);
 }
 
 int https_send_dns_query(https_ctx_t *ctx, const uint8_t *buf, const size_t buf_len)
@@ -532,7 +543,6 @@ int https_recv_dns_response(https_ctx_t *ctx, uint8_t *buf, const size_t buf_len
 		pthread_mutex_unlock(&ctx->recv_mx);
 		return KNOT_NET_ERECV;
 	}
-
 	ctx->recv_buf = NULL;
 
 	pthread_mutex_unlock(&ctx->recv_mx);
@@ -542,7 +552,8 @@ int https_recv_dns_response(https_ctx_t *ctx, uint8_t *buf, const size_t buf_len
 		return KNOT_NET_ERECV;
 	}
 
-	return ctx->recv_buflen;
+	assert(buf_len >= ctx->recv_buflen);
+	return buf_len - ctx->recv_buflen;
 }
 
 void https_ctx_deinit(https_ctx_t *ctx)
