@@ -406,16 +406,14 @@ int interactive_loop(params_t *process_params)
 		const char **argv;
 		const LineInfo *li = el_line(el);
 		int ret = tok_line(tok, li, &argc, &argv, NULL, NULL);
-		if (ret != 0 || argc == 0) {
-			continue;
+		if (ret == 0 && argc != 0) {
+			history(hist, &hev, H_ENTER, line);
+			history(hist, &hev, H_SAVE, hist_file);
+
+			// Process the command.
+			ret = process_cmd(argc, argv, process_params);
 		}
 
-		history(hist, &hev, H_ENTER, line);
-		history(hist, &hev, H_SAVE, hist_file);
-
-		// Process the command.
-		ret = process_cmd(argc, argv, process_params);
-
 		tok_reset(tok);
 		tok_end(tok);
 
